@@ -10,7 +10,7 @@ package Algorithm::AM;
 use strict;
 use warnings;
 # ABSTRACT: Perl extension for Analogical Modeling using a parallel algorithm
-our $VERSION = '2.35'; # TRIAL VERSION;
+our $VERSION = '2.36'; # TRIAL VERSION;
 use feature 'state';
 use Path::Tiny;
 use Exporter::Easy (
@@ -56,9 +56,14 @@ sub new {
 
     #TODO: what is the purpose of these two statements?
     my $class = ref($proto) || $proto;
+    #TODO: this is not a legal path anyway...
     $project_path = ''
         if $proto =~ /^-/;
 
+    if(!$project_path){
+        croak 'Must specify project';
+    }
+
     # all of the options except commas are for the Project object,
     # but creating the Project first could use a lot of time, and
     # incorrect classification options is a fatal error. So put this
@@ -342,7 +347,7 @@ Algorithm::AM - Perl extension for Analogical Modeling using a parallel algorith
 
 =head1 VERSION
 
-version 2.35
+version 2.36
 
 =head1 AUTHOR
 
@@ -359,6 +364,8 @@ the same terms as the Perl 5 programming language system itself.
 
 __DATA__
 
+# line 1000 "start eval"
+
 #print to amcpresults file instead of to the screen
 #TODO: move file choice to different module, and use Log::Any in this one.
 $logger->remove('Screen');
@@ -377,13 +384,12 @@ $self->{beginhook}->($self, $data);
 
 my $left = scalar $project->num_test_items;
 foreach my $item_number (0 .. $project->num_test_items - 1) {
+# line 1100 "loop items"
     $logger->debug("Test items left: $left");
     --$left;
     my $t = $project->get_test_item($item_number);
     ( $curTestOutcome, $data->{curTestItem}, $data->{curTestSpec} ) =
         @$t;
-    # set to index outcomelist instead of actual outcome string
-    $curTestOutcome = $project->short_outcome_index($curTestOutcome);
     # activeVar is the number of active variables; if we exclude nulls,
     # then we need to minus the number of '=' found in this test item;
     # otherwise, it's just the number of columns in a single item vector
@@ -417,6 +423,7 @@ foreach my $item_number (0 .. $project->num_test_items - 1) {
 
     $pass = 0;
     while ( $pass < $self->{repeat} ) {
+# line 1200 "repeat"
         $self->{beginrepeathook}->($self, $data);
         $data->{datacap} = int($data->{datacap});
 
@@ -434,6 +441,7 @@ foreach my $item_number (0 .. $project->num_test_items - 1) {
         }
 
         for ( my $i = $data->{datacap} ; $i ; ) {
+# line 1300 "data hook"
             --$i;
             # skip this data item if the datahook returns false
             # TODO: this and the next one below would be better in an if
@@ -475,6 +483,7 @@ foreach my $item_number (0 .. $project->num_test_items - 1) {
                 $self->{subtooutcome}->{$context} = $outcome;
             }
         }
+# line 1400 "given"
         if ( exists $self->{subtooutcome}->{$nullcontext} ) {
             ++$testindata;
 ## begin exclude given
@@ -482,12 +491,14 @@ foreach my $item_number (0 .. $project->num_test_items - 1) {
             delete $self->{subtooutcome}->{$nullcontext}, ++$self->{eg} if $self->{exclude_given};
 ## end exclude given;
         }
+# line 1500 "print summary"
 
         #TODO: choose Nulls and Gang value here instead of in regex for eval string
         $self->print_summary($data);
         $logger->info('Test item is in the data.')
           if $testindata;
 
+# line 1600 "call XS"
         $self->_fillandcount(X);
         $grandtotal = $self->{pointers}->{'grandtotal'};
         unless ($grandtotal) {
@@ -504,6 +515,7 @@ foreach my $item_number (0 .. $project->num_test_items - 1) {
         my $outcome_format = $project->outcome_format;
         my $data_format = $project->data_format;
 
+# line 1700 "calculate results"
         #TODO: put all of this information in a return value or something!
         $data->{pointermax}    = "";
         $logger->info('Statistical Summary');
@@ -539,6 +551,7 @@ foreach my $item_number (0 .. $project->num_test_items - 1) {
         }
 
 ## begin analogical set
+# line 1800 "analogical set"
         my @datalist = ();
         foreach my $k ( keys %{$self->{pointers}} ) {
             my $p = $self->{pointers}->{$k};
@@ -569,6 +582,7 @@ foreach my $item_number (0 .. $project->num_test_items - 1) {
 ## end analogical set
 
 ## begin gang
+# line 1900 "start gangs"
         #TODO: explain the magic below
         $logger->info('Gang effects');
         my $dashes = '-' x ( $longest + 10 );
@@ -625,6 +639,7 @@ foreach my $item_number (0 .. $project->num_test_items - 1) {
                     )
                 );
 ## begin skip gang list
+# line 2000 "skip gang list"
                 my $i;
                 for (
                     $i = $self->{itemcontextchainhead}->{$k} ;
@@ -705,6 +720,7 @@ foreach my $item_number (0 .. $project->num_test_items - 1) {
     }
     $self->{endtesthook}->($self, $data);
 }
+# line 2100 "end eval"
 
 ( $sec, $min, $hour ) = localtime();
 $logger->info( sprintf( "Time: %2s:%02s:%02s", $hour, $min, $sec ) );
